package com.grack.hdcp;

import Jampack.JampackException;
import Jampack.Solve;
import Jampack.Z;
import Jampack.Zmat;

public class Conspiracy {

	private HdcpDevice[] devices;

	private HdcpSystem system;

	public Conspiracy(HdcpSystem system, HdcpDevice[] devices) {
		super();

		this.system = system;
		this.devices = devices;
	}

	public boolean isConspiracyConsistent() {
		for (int i = 0; i < system.getKeyArraySize(); i++)
			for (int j = 0; j < system.getKeyArraySize(); j++) {
				long key1 = devices[i].calculateKey(devices[j].getPublicKey());
				long key2 = devices[j].calculateKey(devices[i].getPublicKey());
				if (key1 != key2) {
					System.out.println(String.format("%d--%d, %d != %d", i, j,
							key1, key2));
					return false;
				}
			}
		return true;
	}

	public HdcpDevice solveForDevice(PublicKey publicKey)
			throws JampackException {
		return new HdcpDevice(system, solveForSecretKey(publicKey), publicKey);
	}

	private SecretKey solveForSecretKey(PublicKey publicKey)
			throws JampackException {
		Zmat z1 = new Zmat(system.getKeyArraySize(), system.getKeyArraySize());
		Zmat z2 = new Zmat(system.getKeyArraySize(), 1);

		for (int row = 0; row < system.getKeyArraySize(); row++) {
			long rowSolution = devices[row].calculateKey(publicKey);
			int[] publicKeyItems = devices[row].getPublicKey()
					.getPublicKeyItems();

			for (int col = 0; col < publicKeyItems.length; col++) {
				z1.put(row + 1, publicKeyItems[col], new Z(1));
			}

			// System.out.println(rowSolution);
			z2.put(row + 1, 1, new Z(rowSolution));
		}

		// for (int i = 0; i < system.getKeyArraySize(); i++) {
		// for (int j = 0; j < system.getKeyArraySize(); j++) {
		// System.out.print(z1.get(i + 1, j + 1).re);
		// System.out.print(" ");
		// }
		// System.out.println();
		// }

		Zmat sol = Solve.aib(z1, z2);
		long[] systemSecretKey = new long[system.getKeyArraySize()];
		for (int i = 0; i < systemSecretKey.length; i++)
			systemSecretKey[i] = (long) sol.get(i + 1, 1).re;
		return new SecretKey(systemSecretKey);
	}

	public HdcpKeyGenerator solveForKeyGenerator() throws JampackException {
		SecretKey[] systemPrivateKey = new SecretKey[system.getKeyArraySize()];

		int[] publicKeyItems = new int[1];

		PublicKey key;

		// Generate the secret key rows for the trivial keys
		for (int i = 0; i < system.getKeyArraySize(); i++) {
			publicKeyItems[0] = i + 1;

			key = new PublicKey(system, publicKeyItems, true);
			systemPrivateKey[i] = solveForSecretKey(key);
			System.out.println(systemPrivateKey[i]);
		}

		return new HdcpKeyGenerator(system, systemPrivateKey);
	}
}
