package com.grack.hdcp;

public class PublicKey {
	private long publicKey;
	private HdcpSystem system;
	
	public PublicKey( HdcpSystem system, long publicKey ) {
		super();
		this.system = system;
		this.publicKey = publicKey;

		if (publicKey > (1L << (system.getKeyArraySize() - 1)))
			throw new ArrayIndexOutOfBoundsException("Public key value too large");
	}

	public PublicKey( HdcpSystem system,int[] publicKeyItems ) {
		this(system, publicKeyItems, false);
	}

	public PublicKey( HdcpSystem system, int[] publicKeyItems, boolean ignoreHammingWeight ) {
		super();
		this.system = system;
		if (!ignoreHammingWeight && publicKeyItems.length != system.getKeyArraySize() >> 1)
			throw new ArrayIndexOutOfBoundsException("Public key array is the wrong size");
			
		for ( long publicKeyItem : publicKeyItems ) {
			if (publicKeyItem > system.getKeyArraySize())
				throw new ArrayIndexOutOfBoundsException("Public key item too large");
			this.publicKey |= ( 1L << ( publicKeyItem - 1 ) );
		}
	}
		
	public long getPublicKey() {
		return publicKey;
	}
	
	public int[] getPublicKeyItems() {
		int[] items = new int[ system.getKeyArraySize() >> 1 ];
		int index = 0;
		
		for ( int i = 0; i < 64; i++ )
			if ( ( publicKey & ( 1L << i ) ) != 0 )
				items[ index++ ] = i + 1;
		
		return items;
	}
	
	public String toString() {
		return ((Long)publicKey).toString();
	}
	
	public boolean equals(Object other) {
		PublicKey publicKey = (PublicKey) other;
		return publicKey.publicKey == this.publicKey;
	}
}
