package com.grack.hdcp.tests;

import org.omg.CosNaming.IstringHelper;

import Jampack.JampackException;

import com.grack.hdcp.Conspiracy;
import com.grack.hdcp.HdcpDevice;
import com.grack.hdcp.HdcpKeyGenerator;
import com.grack.hdcp.HdcpSystem;
import com.grack.hdcp.PublicKey;
import com.grack.hdcp.SecretKey;

import junit.framework.TestCase;

public class TestConspiracy extends TestCase {
	public void testConspiracy() throws JampackException {
		HdcpSystem system = new HdcpSystem(4);

		HdcpDevice dev1 = new HdcpDevice(system, new SecretKey(new long[] { 26,
				19, 12, 7 }), new PublicKey(system, new int[] { 1, 2 }));
		HdcpDevice dev2 = new HdcpDevice(system, new SecretKey(new long[] { 13,
				13, 22, 5 }), new PublicKey(system, new int[] { 2, 4 }));
		HdcpDevice dev3 = new HdcpDevice(system, new SecretKey(new long[] { 22,
				16, 5, 19 }), new PublicKey(system, new int[] { 1, 3 }));
		HdcpDevice dev4 = new HdcpDevice(system, new SecretKey(new long[] { 9,
				10, 15, 17 }), new PublicKey(system, new int[] { 2, 3 }));

		PublicKey publicKey = new PublicKey(system, new int[] { 1, 4 });

		Conspiracy conspiracy = new Conspiracy(system, new HdcpDevice[] { dev1,
				dev2, dev3, dev4 });

		HdcpDevice solved = conspiracy.solveForDevice(publicKey);
		assertEquals(24, solved.getSecretKey().getKey()[0]);
		assertEquals(9, solved.getSecretKey().getKey()[1]);
		assertEquals(17, solved.getSecretKey().getKey()[2]);
		assertEquals(9, solved.getSecretKey().getKey()[3]);

		assertEquals(dev1.calculateKey(solved.getPublicKey()), solved
				.calculateKey(dev1.getPublicKey()));
		assertEquals(dev2.calculateKey(solved.getPublicKey()), solved
				.calculateKey(dev2.getPublicKey()));
		assertEquals(dev3.calculateKey(solved.getPublicKey()), solved
				.calculateKey(dev3.getPublicKey()));
		assertEquals(dev4.calculateKey(solved.getPublicKey()), solved
				.calculateKey(dev4.getPublicKey()));
	}

	public void testSystemIntegrity() {
		HdcpSystem system = new HdcpSystem(4);

		HdcpDevice dev1 = new HdcpDevice(system, new SecretKey(new long[] { 26,
				19, 12, 7 }), // 31
				new PublicKey(system, new int[] { 1, 2 }));
		HdcpDevice dev2 = new HdcpDevice(system, new SecretKey(new long[] { 13,
				13, 22, 5 }), // 35
				new PublicKey(system, new int[] { 2, 4 }));
		HdcpDevice dev3 = new HdcpDevice(system, new SecretKey(new long[] { 22,
				16, 5, 19 }), // 21
				new PublicKey(system, new int[] { 1, 3 }));
		HdcpDevice dev4 = new HdcpDevice(system, new SecretKey(new long[] { 12,
				19, 9, 16 }), new PublicKey(system, new int[] { 2, 3 }));

		HdcpDevice[] devices = new HdcpDevice[] { dev1, dev2, dev3, dev4 };

		Conspiracy conspiracy = new Conspiracy(system, devices);
		assertTrue(conspiracy.isConspiracyConsistent());
	}

	public void testSystemIntegrity2() {
		HdcpSystem system = new HdcpSystem(4);

		HdcpDevice dev1 = new HdcpDevice(system, new SecretKey(new long[] { 26,
				19, 12, 7 }), new PublicKey(system, new int[] { 1, 2 }));
		HdcpDevice dev2 = new HdcpDevice(system, new SecretKey(new long[] { 12,
				19, 9, 16 }), new PublicKey(system, new int[] { 2, 3 }));
		HdcpDevice dev3 = new HdcpDevice(system, new SecretKey(new long[] { 9,
				10, 15, 17 }), new PublicKey(system, new int[] { 3, 4 }));
		HdcpDevice dev4 = new HdcpDevice(system, new SecretKey(new long[] { 23,
				10, 18, 8 }), new PublicKey(system, new int[] { 4, 1 }));

		HdcpDevice[] devices = new HdcpDevice[] { dev1, dev2, dev3, dev4 };

		Conspiracy conspiracy = new Conspiracy(system, devices);
		assertTrue(conspiracy.isConspiracyConsistent());
	}

	public void testConspiracyForSystem() throws JampackException {
		HdcpSystem system = new HdcpSystem(4);

		HdcpDevice dev1 = new HdcpDevice(system, new SecretKey(new long[] { 26,
				19, 12, 7 }), new PublicKey(system, new int[] { 1, 2 }));
		HdcpDevice dev2 = new HdcpDevice(system, new SecretKey(new long[] { 13,
				13, 22, 5 }), new PublicKey(system, new int[] { 2, 4 }));
		HdcpDevice dev3 = new HdcpDevice(system, new SecretKey(new long[] { 22,
				16, 5, 19 }), new PublicKey(system, new int[] { 1, 3 }));
		HdcpDevice dev4 = new HdcpDevice(system, new SecretKey(new long[] { 12,
				19, 9, 16 }), new PublicKey(system, new int[] { 2, 3 }));

		Conspiracy conspiracy = new Conspiracy(system, new HdcpDevice[] { dev1,
				dev2, dev3, dev4 });

		HdcpDevice dev1a = conspiracy.solveForDevice(dev1.getPublicKey());
		assertEquals(26, dev1a.getSecretKey().getKey()[0]);
		assertEquals(19, dev1a.getSecretKey().getKey()[1]);
		assertEquals(12, dev1a.getSecretKey().getKey()[2]);
		assertEquals(7, dev1a.getSecretKey().getKey()[3]);

		HdcpDevice dev2a = conspiracy.solveForDevice(dev2.getPublicKey());
		assertEquals(13, dev2a.getSecretKey().getKey()[0]);
		assertEquals(13, dev2a.getSecretKey().getKey()[1]);
		assertEquals(22, dev2a.getSecretKey().getKey()[2]);
		assertEquals(5, dev2a.getSecretKey().getKey()[3]);

		assertEquals(dev1.calculateKey(dev1a.getPublicKey()), dev1a
				.calculateKey(dev1.getPublicKey()));
		assertEquals(dev2.calculateKey(dev1a.getPublicKey()), dev1a
				.calculateKey(dev2.getPublicKey()));
		assertEquals(dev3.calculateKey(dev1a.getPublicKey()), dev1a
				.calculateKey(dev3.getPublicKey()));
		assertEquals(dev4.calculateKey(dev1a.getPublicKey()), dev1a
				.calculateKey(dev4.getPublicKey()));

		HdcpKeyGenerator keyGen = conspiracy.solveForKeyGenerator();
		HdcpDevice dev1b = keyGen.createDevice(new PublicKey(system, new int[] {
				1, 2 }));
		assertEquals(26, dev1b.getSecretKey().getKey()[0]);
		assertEquals(19, dev1b.getSecretKey().getKey()[1]);
		assertEquals(12, dev1b.getSecretKey().getKey()[2]);
		assertEquals(7, dev1b.getSecretKey().getKey()[3]);
	}

	public void testLargerConspiracy() throws JampackException {
		HdcpSystem system = new HdcpSystem(10);
		HdcpKeyGenerator keygen = new HdcpKeyGenerator(system, new SecretKey[] {
				new SecretKey(new long[] { 1, 2, 3, 1, 1, 1, 1, 1, 1, 9 }),
				new SecretKey(new long[] { 2, 2, 2, 2, 2, 2, 8, 2, 2, 2 }),
				new SecretKey(new long[] { 3, 2, 3, 3, 5, 6, 7, 8, 9, 9 }),
				new SecretKey(new long[] { 1, 2, 3, 4, 5, 6, 7, 8, 9, 9 }),
				new SecretKey(new long[] { 1, 2, 5, 5, 5, 6, 7, 8, 9, 9 }),
				new SecretKey(new long[] { 1, 2, 6, 6, 6, 6, 6, 6, 6, 6 }),
				new SecretKey(new long[] { 1, 8, 7, 7, 7, 6, 7, 7, 7, 7 }),
				new SecretKey(new long[] { 1, 2, 8, 8, 8, 6, 7, 8, 8, 8 }),
				new SecretKey(new long[] { 1, 2, 9, 9, 9, 6, 7, 8, 9, 9 }),
				new SecretKey(new long[] { 9, 2, 9, 9, 9, 6, 7, 8, 9, 10 }), });

		HdcpDevice dev1 = keygen.createDevice(new PublicKey(system, new int[] {
				1, 2, 3, 4, 5 }));
		HdcpDevice dev2 = keygen.createDevice(new PublicKey(system, new int[] {
				2, 3, 4, 5, 6 }));
		HdcpDevice dev3 = keygen.createDevice(new PublicKey(system, new int[] {
				3, 4, 5, 6, 7 }));
		HdcpDevice dev4 = keygen.createDevice(new PublicKey(system, new int[] {
				4, 5, 6, 7, 8 }));
		HdcpDevice dev5 = keygen.createDevice(new PublicKey(system, new int[] {
				1, 2, 3, 4, 9 }));

		HdcpDevice dev6 = keygen.createDevice(new PublicKey(system, new int[] {
				1, 2, 3, 4, 10 }));
		HdcpDevice dev7 = keygen.createDevice(new PublicKey(system, new int[] {
				1, 2, 3, 5, 6 }));
		HdcpDevice dev8 = keygen.createDevice(new PublicKey(system, new int[] {
				1, 2, 3, 5, 7 }));
		HdcpDevice dev9 = keygen.createDevice(new PublicKey(system, new int[] {
				1, 2, 3, 5, 8 }));
		HdcpDevice dev10 = keygen.createDevice(new PublicKey(system, new int[] {
				1, 2, 3, 5, 9 }));

		Conspiracy conspiracy = new Conspiracy(system, new HdcpDevice[] { dev1,
				dev2, dev3, dev4, dev5, dev6, dev7, dev8, dev9, dev10 });

		assertTrue(conspiracy.isConspiracyConsistent());

		PublicKey pubKey = new PublicKey(system, new int[] { 1, 3, 5, 7, 9 });
		HdcpDevice solved = conspiracy.solveForDevice(pubKey);
		System.out.println(solved);

		assertEquals(solved, keygen.createDevice(pubKey));
	}
}
